/*  
     The _3j procedure was derived from the C program described by William J. Thompson.
          William J. Thompson, Angular Momentum: An Illustrated Guide 
               to Rotational Symmetries for Physical Systems,
                    John Wiley and Sons Inc., 1994

     This program calculates Clebsch-Gordan coefficients
               <j1,j2;m1,m2|j1,j2;J,M>
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "am.h"

typedef struct fraction
 {
  int numer;  /* numerator */
  int denom;  /* denominator */
  char s;     /* sign of the fraction */
 } frac;


double _3j(int ta, int tb,
           int tc, int tma,
           int tmb, int tmc)
           
  {
    double n1, n2, n3, d1, d2, d3;
    double norm, sum, phase, term;
    int minmax[3], tkmin, tkmax, tk;
    
  /*  Normalization Factorials  */
  
     n1 = Fctrl((tc+ta-tb)/2)*Fctrl((tc-ta+tb)/2);
     n2 = Fctrl((ta+tb-tc)/2);
     n3 = Fctrl((tc-tmc)/2)*Fctrl((tc+tmc)/2);
     d1 = Fctrl((ta+tb+tc)/2+1);
     d2 = Fctrl((ta-tma)/2)*Fctrl((ta+tma)/2);
     d3 = Fctrl((tb-tmb)/2)*Fctrl((tb+tmb)/2);
     norm = Power(-1.0,(ta-tb-tmc)/2)*
                   sqrt(n1*n2*n3/(d1*d2*d3));
     
     minmax[1] = 0;
     minmax[2] = tb-ta-tmc;
     tkmin = Max(minmax,2); 
     minmax[1] = tc-ta+tb; 
     minmax[2] = tc-tmc;   
     tkmax = Min(minmax,2);
     sum = 0;
     phase = Power(-1.0, (tkmin+tb+tmb)/2);
      
     for(tk= tkmin; tk <= tkmax; tk = tk+2)
       {
         n1 = Fctrl((tb+tc+tma-tk)/2);
         n2 = Fctrl((ta-tma+tk)/2);
         d1 = Fctrl(tk/2)*Fctrl((tc-ta+tb-tk)/2);  
         d2 = Fctrl((tc-tmc-tk)/2);
         d3 = Fctrl((tk+ta-tb+tmc)/2);
         term = phase*n1*n2/(d1*d2*d3); 
         phase = -phase; 
         sum = sum+norm*term; 
       };
    
     return sum;
     
  }


int EvenOrOdd(int tj, int tm)
  {
    int sum;
    sum = tj+tm; 
    if (sum != 2*(sum/2)) return 1;
    else return 0;
  }  

/* 
   this function tries to convert a decimal number
   into a "reasonable" fraction; 7/14157 not! ... 
*/ 
frac Dec2Frac(double N)
  {  
    int x,y,i,j; /* x & y are temp variable, and i,j are indices */
	int found;       /* a boolean variable */  
    frac num;     /* variable of type frac */
	double CG=N;

    if (CG < 0.0) { num.s ='-' ;} else { num.s='+' ;};
	CG = (CG*CG); 
    x = round(CG * 1E6);
	i = 1; j = 1; found = 0;
	if (x == 0) found = 1;
    while (i <= 100 && !found) 
	 {
	  j=i; 
      while(j <= 100 && !found)  
	   {
	    y = round((i*1.0/j*1.0)*1E6); 
		if (x == y) found = 1;
        j++; 
	   }; 
      i++;
	 };
	i--; j--;

    if (!found) i=-1;  /* did not find a reasonable fraction */
    num.numer=i; num.denom=j;
    return num;

	
  }


int main(void)
  {
        double a,b,c,ma,mb,mc, Three_j,CG;
        int ta,tb,tc,tma,tmb,tmc;
	    frac f;
       
        printf("Clebsch-Gordan coefficient\n");
        printf("\n              Note:    [\xF1\x61] \xF0 \xF1\xFB\x61 \n");

        ta=0;
        while (ta >= 0)
          {
		        printf("\nThe Clebsh-Gordan coefficient: {j1,j2,J,m1,m2,M}");
                printf("\n\nInput j1 as a decimal or -1 to end): ");
                scanf("%lf",&a);
                if (a < 0)
              {
                        printf("\nEnd of CG coefficients\n");
                        exit (0);
                  }
                printf("\nInput j2, J, m1, m2 and M(separated by commas): ");
                scanf("%lf,%lf,%lf,%lf,%lf",&b,&c,&ma,&mb,&mc);

                /* Convert to integers that are twice the input values */

                ta = Twice(a); tb = Twice(b); tc = Twice(c);
                tma = Twice(ma); tmb = Twice(mb); tmc = -1.0*Twice(mc);

           /* Testing for zero coefficient  */

           if (tma+tmb+tmc != 0)
                 {
                   printf("\n!! ma+mb+mc not zero; try again");
                 }
           else
         {
           if (TriangleBroken(ta,tb,tc) !=0) printf(" try again");
           else
             {
               if (abs(tma) > ta || abs(tmb) > tb || abs(tmc) > tc)
                 printf("\n!! An m values is too big; try again");
               else
                 {
                   if (EvenOrOdd(ta,tma) || EvenOrOdd(tb,tmb) || EvenOrOdd(tc,tmc) > 0)
                     printf("\n!! An m does not match a j; try again");
                   else
                     {
					   Three_j = _3j(ta,tb,tc,tma,tmb,tmc);
                       CG = pow((-1.0),(-a+b-mc))*pow((2.0*c + 1.0),0.5)*Three_j;
					   f = Dec2Frac(CG);
					   printf("\nThe Clebsh-Gordan coefficient: ");
                       if (f.numer == -1)
                        {
                         printf("{%.1lf,%.1lf,%.1lf,%.1lf,%.1lf,%.1lf} = %lf",a,b,c,ma,mb,mc,CG);
                        }
					   else
						 { 
                          if (f.numer==0 || f.numer==f.denom)
                           { 
		                    printf("{%.1lf,%.1lf,%.1lf,%.1lf,%.1lf,%.1lf} = %i \n\n", a,b,c,ma,mb,mc,f.numer);
                           }
                          else
                           {
      	                   printf("{%.1lf,%.1lf,%.1lf,%.1lf,%.1lf,%.1lf} = [%c%i/%i] = %lf \n\n", a,b,c,ma,mb,mc,f.s,f.numer,f.denom, CG);
	                       };
						 }
                     }
                 }
             }
         }
      }
  }    
                

                
